@@ -34,6 +34,25 @@ impl StateHub {
         }
     }
 
+    pub(super) async fn remove_location_helper(
+        &self,
+        name: &v1::StateName,
+        location: &Location,
+    ) -> anyhow::Result<()> {
+        match location {
+            Location::Aws(region) => self
+                .api
+                .del_aws_location(name.clone(), *region)
+                .await
+                .map(|_aws| ()),
+            Location::Azure(region) => self
+                .api
+                .del_azure_location(name.clone(), *region)
+                .await
+                .map(|_azure| ()),
+        }
+    }
+
     pub(super) async fn adjust_all_states(
         &self,
         names: &[v1::StateName],
@@ -67,6 +86,21 @@ impl StateHub {
         Ok(())
     }
 
+    pub(super) async fn setup_configmap_helper(
+        &self,
+        cluster: &v1::Cluster,
+        helm: &k8s::Helm,
+    ) -> anyhow::Result<()> {
+        let token = self.api.issue_cluster_token(&cluster.name).await?;
+        self.verbosely(format!("Issued token {} for {}", token.token, cluster));
+        let namespace = helm.namespace();
+
+        let default_storage_class = helm.default_storage_class().unwrap_or("");
+
+        k8s::store_configmap(namespace, &cluster.name, default_storage_class).await?;
+        Ok(())
+    }
+
     pub(super) async fn setup_cluster_token_helper(
         &self,
         cluster: &v1::Cluster,
