@@ -16,6 +16,9 @@ use crate::v1;
 use crate::Location;
 use crate::Output;
 
+use config::Config;
+
+mod config;
 mod helper;
 
 const ABOUT: &str = "statehub CLI tool";
@@ -25,12 +28,12 @@ const ABOUT: &str = "statehub CLI tool";
 pub(crate) struct Cli {
     #[structopt(
         help = "Management server URL or address",
-        default_value = "https://api.statehub.io",
+        // default_value = "https://api.statehub.io",
         short,
         long,
         env = "SHAPI"
     )]
-    management: String,
+    management: Option<String>,
     #[structopt(help = "Authentication token", short, long, env = "SHTOKEN")]
     token: Option<String>,
     #[structopt(long, global = true, help = "Show results as JSON")]
@@ -43,7 +46,7 @@ pub(crate) struct Cli {
 
 #[derive(Debug, StructOpt)]
 enum Command {
-    #[structopt(about = "Create new state", aliases = &["create-st", "cs"])]
+    #[structopt(about = "Create new state", aliases = &["create-st", "cs"], display_order(20))]
     CreateState {
         #[structopt(help = "State name")]
         name: v1::StateName,
@@ -52,21 +55,26 @@ enum Command {
         #[structopt(long, short, help = "Location definition")]
         location: Vec<Location>,
     },
-    #[structopt(about = "Delete existing state", aliases = &["delete-st", "ds"])]
+
+    #[structopt(about = "Delete existing state", aliases = &["delete-st", "ds"], display_order(20))]
     DeleteState {
         #[structopt(help = "State name")]
         name: v1::StateName,
     },
-    #[structopt(about = "List available states", aliases = &["list-state", "list-st", "ls"])]
+
+    #[structopt(about = "List available states", aliases = &["list-state", "list-st", "ls"], display_order(20))]
     ListStates,
-    #[structopt(about = "Show state details", aliases = &["show-s", "ss"])]
+
+    #[structopt(about = "Show state details", aliases = &["show-s", "ss"], display_order(20))]
     ShowState {
         #[structopt(help = "State name")]
         name: v1::StateName,
     },
-    #[structopt(about = "List registered clusters", aliases = &["list-cluster", "list-cl", "lc"])]
+
+    #[structopt(about = "List registered clusters", aliases = &["list-cluster", "list-cl", "lc"], display_order(10))]
     ListClusters,
-    #[structopt(about = "Register new cluster", aliases = &["register-cl", "rc"])]
+
+    #[structopt(about = "Register new cluster", aliases = &["register-cl", "rc"], display_order(10))]
     RegisterCluster {
         #[structopt(help = "Cluster name")]
         name: v1::ClusterName,
@@ -113,60 +121,79 @@ enum Command {
         #[structopt(help = "Skip running 'helm install'", long)]
         skip_helm: bool,
     },
-    #[structopt(about = "Unregister existing cluster", aliases = &["unregister-cl", "uc"])]
+
+    #[structopt(about = "Unregister existing cluster", aliases = &["unregister-cl", "uc"], display_order(10))]
     UnregisterCluster {
         #[structopt(help = "Skip confirmation", long, short)]
         force: bool,
         #[structopt(help = "Cluster name")]
         name: v1::ClusterName,
     },
-    #[structopt(about = "Manually create new volume")]
-    CreateVolume,
-    #[structopt(about = "Manually delete existing volume")]
-    DeleteVolume,
+
     #[structopt(
         about = "Manually make state available in a specified location",
         aliases = &["add-l", "al"],
+        display_order(30)
     )]
     AddLocation {
         #[structopt(help = "State name")]
         state: v1::StateName,
-        #[structopt(help = "Location definition")]
+        #[structopt(help = "Location specification")]
         location: Location,
     },
+
     #[structopt(
         about = "Manually make state unavailable in a specified location",
         aliases = &["remove-l", "rem-l", "rl"],
+        display_order(30)
     )]
-    RemoveLocation,
-    #[structopt(about = "Set state availability grade")]
+    RemoveLocation {
+        #[structopt(help = "State name")]
+        state: v1::StateName,
+        #[structopt(help = "Location specification")]
+        location: Location,
+    },
+
+    #[structopt(about = "Set state availability grade", display_order(40))]
     SetAvailability,
-    #[structopt(about = "Set cluster as the state owner")]
+
+    #[structopt(about = "Set cluster as the state owner", display_order(40))]
     SetOwner {
         #[structopt(help = "State name")]
         state: v1::StateName,
         #[structopt(help = "Cluster name")]
         cluster: v1::ClusterName,
     },
-    #[structopt(about = "Clear state owner")]
+
+    #[structopt(about = "Clear state owner", display_order(40))]
     UnsetOwner {
         #[structopt(help = "State name")]
         state: v1::StateName,
         #[structopt(help = "Cluster name")]
         cluster: v1::ClusterName,
     },
+
+    #[structopt(about = "Manually create new volume", display_order(50))]
+    CreateVolume,
+
+    #[structopt(about = "Manually delete existing volume", display_order(50))]
+    DeleteVolume,
+
     #[structopt(
         about = "Create new namespace",
         alias = "c-ns",
+        display_order(1000),
         // setting(clap::AppSettings::Hidden)
     )]
     CreateNamespace {
         #[structopt(help = "Namespace name")]
         namespace: String,
     },
+
     #[structopt(
         about = "Save cluster token",
         alias = "sct",
+        display_order(1000),
         // setting(clap::AppSettings::Hidden)
     )]
     SaveClusterToken {
@@ -175,27 +202,37 @@ enum Command {
         #[structopt(help = "Cluster token")]
         token: String,
     },
+
     #[structopt(
         about = "List K8s namespaces",
         alias = "list-ns",
+        display_order(1000),
         // setting(clap::AppSettings::Hidden)
     )]
     ListNamespaces,
+
     #[structopt(
         about = "List K8s nodes",
+        display_order(1000),
         // setting(clap::AppSettings::Hidden)
     )]
     ListNodes,
+
     #[structopt(
         about = "List K8s pods",
+        display_order(1000),
         // setting(clap::AppSettings::Hidden)
     )]
     ListPods,
-    #[structopt(about = "List K8s node regions")]
+
+    #[structopt(about = "List K8s node regions", display_order(1000))]
     ListRegions {
         #[structopt(help = "Also list zones", long, short)]
         zone: bool,
     },
+
+    #[structopt(about = "Save default configuration file", display_order(2000))]
+    SaveConfig,
 }
 
 impl Cli {
@@ -203,8 +240,20 @@ impl Cli {
         Self::from_args().dispatch().await
     }
 
+    async fn config(&self) -> anyhow::Result<Config> {
+        Config::load()
+    }
+
     async fn dispatch(self) -> anyhow::Result<()> {
-        let statehub = StateHub::new(self.management, self.token, self.json, self.verbose);
+        let config = self
+            .config()
+            .await?
+            .optionally_management_api(self.management)
+            .set_token(self.token);
+
+        let statehub = StateHub::new(config, self.json, self.verbose);
+
+        statehub.validate_auth().await?;
 
         match self.command {
             Command::CreateState {
@@ -251,15 +300,17 @@ impl Cli {
             Command::UnregisterCluster { force, name } => {
                 statehub.unregister_cluster(name, force).await
             }
-            Command::CreateVolume => statehub.create_volume().await,
-            Command::DeleteVolume => statehub.delete_volume().await,
             Command::AddLocation { state, location } => {
                 statehub.add_location(state, location).await
             }
-            Command::RemoveLocation => statehub.remove_location().await,
+            Command::RemoveLocation { state, location } => {
+                statehub.remove_location(state, location).await
+            }
             Command::SetAvailability => statehub.set_availability().await,
             Command::SetOwner { state, cluster } => statehub.set_owner(state, cluster).await,
             Command::UnsetOwner { state, cluster } => statehub.unset_owner(state, cluster).await,
+            Command::CreateVolume => statehub.create_volume().await,
+            Command::DeleteVolume => statehub.delete_volume().await,
             Command::CreateNamespace { namespace } => statehub.create_namespace(namespace).await,
             Command::SaveClusterToken { namespace, token } => {
                 statehub.save_cluster_token(namespace, token).await
@@ -268,21 +319,37 @@ impl Cli {
             Command::ListNodes => statehub.list_nodes().await,
             Command::ListPods => statehub.list_pods().await,
             Command::ListRegions { zone } => statehub.list_regions(zone).await,
+            Command::SaveConfig => statehub.save_config().await,
         }
     }
 }
 
 pub(crate) struct StateHub {
+    config: Config,
     api: api::Api,
     json: bool,
     verbose: bool,
 }
 
 impl StateHub {
-    fn new(management: String, token: Option<String>, json: bool, verbose: bool) -> Self {
-        let api = api::Api::new(management, token, verbose);
+    fn new(config: Config, json: bool, verbose: bool) -> Self {
+        let api = api::Api::new(config.api(), config.token(), verbose);
 
-        Self { api, json, verbose }
+        Self {
+            config,
+            api,
+            json,
+            verbose,
+        }
+    }
+
+    async fn validate_auth(&self) -> anyhow::Result<()> {
+        if self.api.is_unauthorized().await {
+            // log::error!("Unauthorized - perhaps an invalid token?");
+            anyhow::bail!("Unauthorized - perhaps an invalid token?");
+        }
+
+        Ok(())
     }
 
     pub(crate) async fn create_state(
@@ -342,6 +409,7 @@ impl StateHub {
         k8s::validate_namespace(helm.namespace()).await?;
 
         self.setup_cluster_token_helper(&output, &helm).await?;
+        self.setup_configmap_helper(&output, &helm).await?;
 
         helm.execute(&output, self.verbose).await?;
 
@@ -376,6 +444,10 @@ impl StateHub {
         self.add_location_helper(&state, &location).await
     }
 
+    async fn remove_location(self, state: v1::StateName, location: Location) -> anyhow::Result<()> {
+        self.remove_location_helper(&state, &location).await
+    }
+
     pub(crate) async fn create_volume(self) -> anyhow::Result<()> {
         // Ok(Output::<String>::todo()).handle_output(self.json)
         anyhow::bail!(self.show(Output::<String>::todo()))
@@ -386,11 +458,6 @@ impl StateHub {
         anyhow::bail!(self.show(Output::<String>::todo()))
     }
 
-    pub(crate) async fn remove_location(self) -> anyhow::Result<()> {
-        // Ok(Output::<String>::todo()).handle_output(self.json)
-        anyhow::bail!(self.show(Output::<String>::todo()))
-    }
-
     pub(crate) async fn set_availability(self) -> anyhow::Result<()> {
         // Ok(Output::<String>::todo()).handle_output(self.json)
         anyhow::bail!(self.show(Output::<String>::todo()))
@@ -437,6 +504,22 @@ impl StateHub {
             .map(|namespace| println!("{:#?}", namespace))
     }
 
+    /*async fn _send_configmap(
+        &self,
+        namespace: String,
+        cluster_name: v1::ClusterName,
+        default_storage_class: String,
+    ) -> anyhow::Result<()> {
+        let _configmap =
+            k8s::store_configmap(&namespace, &cluster_name, &default_storage_class).await?;
+        //if let Some(token) = k8s::extract_cluster_token(&secret) {
+        //    log::debug!("Token: {}", token);
+        //}
+
+        // TODO: call some api function to send configmap
+        Ok(())
+    }*/
+
     async fn save_cluster_token(&self, namespace: String, token: String) -> anyhow::Result<()> {
         let secret = k8s::store_cluster_token(&namespace, &token).await?;
         if let Some(token) = k8s::extract_cluster_token(&secret) {
@@ -469,6 +552,10 @@ impl StateHub {
         Ok(())
     }
 
+    async fn save_config(&self) -> anyhow::Result<()> {
+        self.config.save()
+    }
+
     fn confirm(&self, text: impl fmt::Display) -> bool {
         print!("{}", text);
         let yes: String = text_io::read!();
